@@ -25,8 +25,7 @@
 
 #include "test.h"
 
-#define __LIBARCHIVE_BUILD 1
-#include "archive_getdate.h"
+#define parse_date archive_parse_date
 
 static void
 test_newer_time(void)
@@ -102,27 +101,27 @@ test_newer_time_str(void)
 
 	/* Test1: Allow newer time. */
 	archive_entry_copy_pathname(ae, "file1");
-	t = __archive_get_date(now, "1980/2/1 0:0:0 UTC");
+	t = parse_date(now, "1980/2/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/2/1 0:0:1 UTC");
+	t = parse_date(now, "1980/2/1 0:0:1 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
 	assertEqualInt(0, archive_match_time_excluded(m, ae));
 	assertEqualInt(0, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/2/1 0:0:0 UTC");
+	t = parse_date(now, "1980/2/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 1);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Its mtime should be excluded");
@@ -143,20 +142,20 @@ test_newer_time_str(void)
 	    "1980/2/1 0:0:0 UTC"));
 
 	archive_entry_copy_pathname(ae, "file1");
-	t = __archive_get_date(now, "1980/2/1 0:0:0 UTC");
+	t = parse_date(now, "1980/2/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
 	assertEqualInt(0, archive_match_time_excluded(m, ae));
 	assertEqualInt(0, archive_match_excluded(m, ae));
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/2/1 0:0:1 UTC");
+	t = parse_date(now, "1980/2/1 0:0:1 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
@@ -190,27 +189,27 @@ test_newer_time_str_w(void)
 
 	/* Test1: Allow newer time. */
 	archive_entry_copy_pathname(ae, "file1");
-	t = __archive_get_date(now, "1980/2/1 0:0:0 UTC");
+	t = parse_date(now, "1980/2/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/2/1 0:0:1 UTC");
+	t = parse_date(now, "1980/2/1 0:0:1 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
 	assertEqualInt(0, archive_match_time_excluded(m, ae));
 	assertEqualInt(0, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/2/1 0:0:0 UTC");
+	t = parse_date(now, "1980/2/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 1);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Its mtime should be excluded");
@@ -231,20 +230,20 @@ test_newer_time_str_w(void)
 	    L"1980/2/1 0:0:0 UTC"));
 
 	archive_entry_copy_pathname(ae, "file1");
-	t = __archive_get_date(now, "1980/2/1 0:0:0 UTC");
+	t = parse_date(now, "1980/2/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
 	assertEqualInt(0, archive_match_time_excluded(m, ae));
 	assertEqualInt(0, archive_match_excluded(m, ae));
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/2/1 0:0:1 UTC");
+	t = parse_date(now, "1980/2/1 0:0:1 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
@@ -316,15 +315,14 @@ test_newer_mtime_than_file_mbs(void)
 static void
 test_newer_ctime_than_file_mbs(void)
 {
+#if defined(_WIN32) && !defined(__CYGWIN__)
+	skipping("Can't set ctime on Windows");
+	return;
+#else
 	struct archive *a;
 	struct archive_entry *ae;
 	struct archive *m;
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
-        skipping("Can't set ctime on Windows");
-        return;
-#endif
-
 	if (!assert((m = archive_match_new()) != NULL))
 		return;
 	if (!assert((ae = archive_entry_new()) != NULL)) {
@@ -373,6 +371,7 @@ test_newer_ctime_than_file_mbs(void)
 	archive_read_free(a);
 	archive_entry_free(ae);
 	archive_match_free(m);
+#endif
 }
 
 static void
@@ -435,15 +434,14 @@ test_newer_mtime_than_file_wcs(void)
 static void
 test_newer_ctime_than_file_wcs(void)
 {
+#if defined(_WIN32) && !defined(__CYGWIN__)
+	skipping("Can't set ctime on Windows");
+	return;
+#else
 	struct archive *a;
 	struct archive_entry *ae;
 	struct archive *m;
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
-        skipping("Can't set ctime on Windows");
-        return;
-#endif
-
 	if (!assert((m = archive_match_new()) != NULL))
 		return;
 	if (!assert((ae = archive_entry_new()) != NULL)) {
@@ -493,6 +491,7 @@ test_newer_ctime_than_file_wcs(void)
 	archive_read_free(a);
 	archive_entry_free(ae);
 	archive_match_free(m);
+#endif
 }
 
 static void
@@ -569,37 +568,37 @@ test_older_time_str(void)
 	    ARCHIVE_MATCH_OLDER, "1980/2/1 0:0:0 UTC"));
 
 	archive_entry_copy_pathname(ae, "file1");
-	t = __archive_get_date(now, "1980/2/1 0:0:0 UTC");
+	t = parse_date(now, "1980/2/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
 	assertEqualInt(0, archive_match_time_excluded(m, ae));
 	assertEqualInt(0, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/3/1 0:0:0 UTC");
+	t = parse_date(now, "1980/3/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/3/1 0:0:0 UTC");
+	t = parse_date(now, "1980/3/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Its mtime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
-	t = __archive_get_date(now, "1980/3/1 0:0:0 UTC");
+	t = parse_date(now, "1980/3/1 0:0:0 UTC");
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Its ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
@@ -612,20 +611,20 @@ test_older_time_str(void)
 	    "1980/2/1 0:0:0 UTC"));
 
 	archive_entry_copy_pathname(ae, "file1");
-	t = __archive_get_date(now, "1980/2/1 0:0:0 UTC");
+	t = parse_date(now, "1980/2/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
 	assertEqualInt(0, archive_match_time_excluded(m, ae));
 	assertEqualInt(0, archive_match_excluded(m, ae));
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
 	assertEqualInt(0, archive_match_time_excluded(m, ae));
 	assertEqualInt(0, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/3/1 0:0:0 UTC");
+	t = parse_date(now, "1980/3/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
@@ -659,37 +658,37 @@ test_older_time_str_w(void)
 	    ARCHIVE_MATCH_OLDER, L"1980/2/1 0:0:0 UTC"));
 
 	archive_entry_copy_pathname(ae, "file1");
-	t = __archive_get_date(now, "1980/2/1 0:0:0 UTC");
+	t = parse_date(now, "1980/2/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
 	assertEqualInt(0, archive_match_time_excluded(m, ae));
 	assertEqualInt(0, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/3/1 0:0:0 UTC");
+	t = parse_date(now, "1980/3/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/3/1 0:0:0 UTC");
+	t = parse_date(now, "1980/3/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Its mtime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
 	assertEqualInt(1, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
-	t = __archive_get_date(now, "1980/3/1 0:0:0 UTC");
+	t = parse_date(now, "1980/3/1 0:0:0 UTC");
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Its ctime should be excluded");
 	assertEqualInt(1, archive_match_time_excluded(m, ae));
@@ -702,20 +701,20 @@ test_older_time_str_w(void)
 	    L"1980/2/1 0:0:0 UTC"));
 
 	archive_entry_copy_pathname(ae, "file1");
-	t = __archive_get_date(now, "1980/2/1 0:0:0 UTC");
+	t = parse_date(now, "1980/2/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
 	assertEqualInt(0, archive_match_time_excluded(m, ae));
 	assertEqualInt(0, archive_match_excluded(m, ae));
-	t = __archive_get_date(now, "1980/1/1 0:0:0 UTC");
+	t = parse_date(now, "1980/1/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should not be excluded");
 	assertEqualInt(0, archive_match_time_excluded(m, ae));
 	assertEqualInt(0, archive_match_excluded(m, ae));
 
-	t = __archive_get_date(now, "1980/3/1 0:0:0 UTC");
+	t = parse_date(now, "1980/3/1 0:0:0 UTC");
 	archive_entry_set_mtime(ae, t, 0);
 	archive_entry_set_ctime(ae, t, 0);
 	failure("Both Its mtime and ctime should be excluded");
@@ -787,15 +786,14 @@ test_older_mtime_than_file_mbs(void)
 static void
 test_older_ctime_than_file_mbs(void)
 {
+#if defined(_WIN32) && !defined(__CYGWIN__)
+	skipping("Can't set ctime on Windows");
+	return;
+#else
 	struct archive *a;
 	struct archive_entry *ae;
 	struct archive *m;
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
-        skipping("Can't set ctime on Windows");
-        return;
-#endif
-
 	if (!assert((m = archive_match_new()) != NULL))
 		return;
 	if (!assert((ae = archive_entry_new()) != NULL)) {
@@ -845,6 +843,7 @@ test_older_ctime_than_file_mbs(void)
 	archive_read_free(a);
 	archive_entry_free(ae);
 	archive_match_free(m);
+#endif
 }
 
 static void
@@ -907,15 +906,14 @@ test_older_mtime_than_file_wcs(void)
 static void
 test_older_ctime_than_file_wcs(void)
 {
+#if defined(_WIN32) && !defined(__CYGWIN__)
+	skipping("Can't set ctime on Windows");
+	return;
+#else
 	struct archive *a;
 	struct archive_entry *ae;
 	struct archive *m;
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
-        skipping("Can't set ctime on Windows");
-        return;
-#endif
-
 	if (!assert((m = archive_match_new()) != NULL))
 		return;
 	if (!assert((ae = archive_entry_new()) != NULL)) {
@@ -965,6 +963,7 @@ test_older_ctime_than_file_wcs(void)
 	archive_read_free(a);
 	archive_entry_free(ae);
 	archive_match_free(m);
+#endif
 }
 
 static void
@@ -1088,15 +1087,14 @@ test_mtime_between_files_wcs(void)
 static void
 test_ctime_between_files_mbs(void)
 {
+#if defined(_WIN32) && !defined(__CYGWIN__)
+	skipping("Can't set ctime on Windows");
+	return;
+#else
 	struct archive *a;
 	struct archive_entry *ae;
 	struct archive *m;
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
-        skipping("Can't set ctime on Windows");
-        return;
-#endif
-
 	if (!assert((m = archive_match_new()) != NULL))
 		return;
 	if (!assert((ae = archive_entry_new()) != NULL)) {
@@ -1147,20 +1145,20 @@ test_ctime_between_files_mbs(void)
 	archive_read_free(a);
 	archive_entry_free(ae);
 	archive_match_free(m);
+#endif
 }
 
 static void
 test_ctime_between_files_wcs(void)
 {
+#if defined(_WIN32) && !defined(__CYGWIN__)
+	skipping("Can't set ctime on Windows");
+	return;
+#else
 	struct archive *a;
 	struct archive_entry *ae;
 	struct archive *m;
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
-        skipping("Can't set ctime on Windows");
-        return;
-#endif
-
 	if (!assert((m = archive_match_new()) != NULL))
 		return;
 	if (!assert((ae = archive_entry_new()) != NULL)) {
@@ -1211,6 +1209,7 @@ test_ctime_between_files_wcs(void)
 	archive_read_free(a);
 	archive_entry_free(ae);
 	archive_match_free(m);
+#endif
 }
 
 static void
